package com.jz.dm.common.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jz.dm.common.exception.OpenApiException;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * openapi请求
 *
 */
public class OpenApiRequest {

    private String              appKey; //apiKey

    private String              openApiParams;

    private JSONObject parameters;

    private Object              parameterObject;

    private Map<String, Object> extAttributes = new HashMap<String, Object>();

    public OpenApiRequest(String openApiParams) {
        setOpenApiParams(openApiParams);
    }

    private void setOpenApiParams(String openApiParams) {
        this.openApiParams = openApiParams;
        try {
            parameters = JSON.parseObject(openApiParams, JSONObject.class);
        } catch (Exception ex) {
            throw new OpenApiException(OpenApiResultCode.ILLEGAL_ARGUMENT,
                "parse openapi params error, openApiParams=" + openApiParams, ex);
        }
    }

    public String getAppKey() {
        return appKey;
    }

    public void setAppKey(String appKey) {
        this.appKey = appKey;
    }

    @SuppressWarnings("unchecked")
    public <T> T getObject(Class<?> clazz) {
        if (parameterObject != null) {
            return (T) parameterObject;
        }
        try {
            this.parameterObject = JSON.parseObject(openApiParams, clazz);
        } catch (Exception ex) {
            throw new OpenApiException(OpenApiResultCode.ILLEGAL_ARGUMENT, ex);
        }
        return (T) parameterObject;
    }

    public String getOpenApiParams() {
        return this.openApiParams;
    }

    @SuppressWarnings("unchecked")
    @Deprecated
    public <T> T getParameter(String key) {
        return (T) parameters.get(key);
    }

    public <T> T getObject(String key, Class<T> clazz) {
        return parameters.getObject(key, clazz);
    }

    public byte[] getBytes(String key) {
        return parameters.getBytes(key);
    }

    public Boolean getBoolean(String key) {
        return parameters.getBoolean(key);
    }

    public boolean getBooleanValue(String key) {
        return parameters.getBooleanValue(key);
    }

    public Byte getByte(String key) {
        return parameters.getByte(key);
    }

    public byte getByteValue(String key) {
        return parameters.getByteValue(key);
    }

    public Short getShort(String key) {
        return parameters.getShortValue(key);
    }

    public short getShortValue(String key) {
        return parameters.getShortValue(key);
    }

    public Integer getInteger(String key) {
        return parameters.getInteger(key);
    }

    public int getIntValue(String key) {
        return parameters.getIntValue(key);
    }

    public Long getLong(String key) {
        return parameters.getLong(key);
    }

    public long getLongValue(String key) {
        return parameters.getLongValue(key);
    }

    public Float getFloat(String key) {
        return parameters.getFloat(key);
    }

    public float getFloatValue(String key) {
        return parameters.getFloatValue(key);
    }

    public Double getDouble(String key) {
        return parameters.getDouble(key);
    }

    public double getDoubleValue(String key) {
        return parameters.getDoubleValue(key);
    }

    public BigDecimal getBigDecimal(String key) {
        return parameters.getBigDecimal(key);
    }

    public BigInteger getBigInteger(String key) {
        return parameters.getBigInteger(key);
    }

    public String getString(String key) {
        return parameters.getString(key);
    }

    public Date getDate(String key) {
        return parameters.getDate(key);
    }

    /**
     * Get Extend Attribute
     * 
     * @param attributeName
     * @return
     */
    @SuppressWarnings("unchecked")
    public <T> T getExtAttribute(String attributeName) {
        return (T) extAttributes.get(attributeName);
    }

    /**
     * Set Extend Attribute
     * 
     * @param attributeName
     * @param attributeValue
     */
    public void setExtAttribute(String attributeName, String attributeValue) {
        this.extAttributes.put(attributeName, attributeValue);
    }

    /**
     * Getter method for property <tt>extAttributes</tt>.
     * 
     * @return property value of extAttributes
     */
    public Map<String, Object> getExtAttributes() {
        return extAttributes;
    }

    /**
     * Setter method for property <tt>extAttributes</tt>.
     * 
     * @param extAttributes value to be assigned to property extAttributes
     */
    public void setExtAttributes(Map<String, Object> extAttributes) {
        this.extAttributes = extAttributes;
    }

}
