package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.req.organizationManage.OrganizationManageAddReq;
import com.jz.dm.models.req.organizationManage.OrganizationManageDetailQueryReq;
import com.jz.dm.models.req.organizationManage.OrganizationManageListQueryReq;
import com.jz.dm.models.req.organizationManage.OrganizationManageUpdateReq;
import com.jz.dm.service.OrganizationManageService;
import com.jz.dm.web.annotation.AccessLimit;
import com.jz.dm.web.annotation.ApiLogAspect;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: OrganizationManageController
 * @DATE: 2020-12-23/22:00
 * @DAY_NAME_SHORT: 周三
 * @Description: 组织管理controller
 **/
@RestController
@RequestMapping("api/organization")
@Api(tags = "组织管理Controller")
public class OrganizationManageController {

    @Autowired
    private OrganizationManageService organizationManageService;

    /**
    * @Description:组织列表查询
    * @return: 组织列表
    * @Author: Mr.zhang
    * @Date: 2020-12-24
    */
    @ApiOperation("组织列表查询")
    @PostMapping(value = "/listOrg")
    public Mono<Result> listOrganization(@RequestBody @Valid OrganizationManageListQueryReq req) {
        return Mono.fromSupplier(() -> Result.of_success(organizationManageService.listOrganization(req)));
    }
    /**
    * @Description:组织信息详情信息查询
    * @return: 组织详情
    * @Author: Mr.zhang
    * @Date: 2020-12-24
    */
    @ApiOperation("组织详情查询")
    @PostMapping(value = "/getOrgDetail")
    @AccessLimit(limit = 1000,sec = 1)
    @ApiLogAspect
    public Mono<Result> getOrganizationDetail(@RequestBody @Valid OrganizationManageDetailQueryReq req) {
        return Mono.fromSupplier(() -> organizationManageService.getOrganizationDetail(req));
    }
    /**
     * @Description:添加组织信息
     * @Author: Mr.zhang
     * @Date: 2020-12-24
     */
    @ApiOperation("添加组织")
    @PostMapping(value = "/add")
    public Mono<Result> add(@RequestBody @Valid OrganizationManageAddReq req) {
        return Mono.fromSupplier(() -> organizationManageService.add(req));
    }
    /**
     * @Description:更新组织信息
     * @Author: Mr.zhang
     * @Date: 2020-12-24
     */
    @ApiOperation("更新组织信息")
    @PostMapping(value = "/update")
    public Mono<Result> update(@RequestBody @Valid OrganizationManageUpdateReq req) {
        return Mono.fromSupplier(() -> organizationManageService.update(req));
    }
    /**
     * @Description:删除组织
     * @Author: Mr.zhang
     * @Date: 2020-12-24
     */
    @ApiOperation("删除组织")
    @GetMapping(value = "/deleted")
    public Mono<Result> delete(@RequestParam(name = "id")Long id) {
        return Mono.fromSupplier(() -> organizationManageService.delete(id));
    }
}
