package com.jz.dm.controller;

import com.jz.common.utils.Result;
import com.jz.dm.models.req.make.*;
import com.jz.dm.service.ProducerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.validation.Valid;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.controller
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ProducerController
 * @DATE: 2020-12-23/16:44
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@RestController
@RequestMapping("api/producer")
@Api(tags = "Api制作Controller")
public class ProducerController {

    @Autowired
    private ProducerService producerService;


    /**
     * @Description:Api制作(自定义)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(获取文件夹列表)")
    @GetMapping(value = "/getFileCatalog")
    public Mono<Result> getFileCatalog(@RequestParam(name = "pageNum",defaultValue = "1",required = false)String pageNum,
                                       @RequestParam(name = "pageSize",defaultValue = "10",required = false)String pageSize) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.getFileCatalog(pageNum,pageSize)));
    }
    /**
     * @Description:Api制作(自定义)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(自定义)")
    @PostMapping(value = "/addCustomApi")
    public Mono<Result> makeCustomApi(@RequestBody @Valid MakeCustomApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addCustomApi(req)));
    }

    /**
     * @Description:Api制作(数据银行)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(数据银行)")
    @PostMapping(value = "/addDataBankApiInfo")
    public Mono<Result> addDataBankApiInfo(@RequestBody @Valid MakeDataBankApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addDataBankApiInfo(req)));
    }

    /**
     * @Description:Api制作(实时接入)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(实时接入)")
    @PostMapping(value = "/realMakeApi")
    public Mono<Result> realCustomApi(@RequestBody @Valid MakeRealCustomApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addRealCustomApi(req)));
    }
    /**
     * @Description:Api制作(大数据查询/标签查询)
     * @return:
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("Api制作(大数据查询/标签查询)")
    @PostMapping(value = "/makeBigDataApi")
    public Mono<Result> bigDataMakeApi(@RequestBody @Valid MakeBigDataApiReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.addBigDataMakeApi(req)));
    }

    /**
     * @Description:保存API基本信息
     * @return: (保存API基本信息)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("保存API基本信息")
    @PostMapping(value = "/saveInterface")
    public Mono<Result> saveInsetInterface(@RequestBody @Valid ApiInterfaceReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.saveInterfaceAPi(req)));
    }
    /**
     * @Description:Api制作(大数据查询/标签查询)保存修改的api信息
     * @return: (自定义更新同时使用)
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("保存修改的api信息")
    @PostMapping(value = "/保存修改的api信息")
    public Mono<Result> saveUpdateAPiInfo(@RequestBody @Valid MakeApiSaveInfoReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.saveUpdateAPiInfo(req)));
    }


    /**
     * @Description:获取数据源配置
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取数据源配置")
    @PostMapping(value = "/getSourceConfig")
    public Mono<Result> getSourceConfig(@RequestBody @Valid SourceConfigReq req) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.getSourceConfig(req)));
    }
    /**
     * @Description:获取索引
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取索引")
    @GetMapping(value = "/getIndicess")
    public Mono<Result> getSourceWarehouseList(@RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.getSourceWarehouseList(id)));
    }
    /**
     * @Description:获取某个索引下的type
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取某个索引下的type")
    @GetMapping(value = "/getIndicesTypes")
    public Mono<Result> getIndicesTypes(@RequestParam(name = "indices") String indices,
                                        @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.getIndicesTypes(indices,id)));
    }
    /**
     * @Description:获取某个索引 type的fields
     * @Author: Mr.zhang
     * @Date: 2020-12-26
     */
    @ApiOperation("获取某个索引 type的fields")
    @GetMapping(value = "/getIndicesTypesFields")
    public Mono<Result> getIndicesTypesFields(@RequestParam(name = "indices") String indices,
                                              @RequestParam(name = "type") String type,
                                              @RequestParam(name = "id") Long id) {
        return Mono.fromSupplier(() -> Result.of_success(producerService.getIndicesTypesFields(indices,type,id)));
    }
}
