package com.jz.dm.filter;

import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.SignType;
import com.jz.dm.common.util.StringUtil;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.models.enity.GatewayRequest;
import com.jz.dm.models.enity.GatewayResponse;
import com.jz.dm.common.enums.Format;
import com.jz.dm.common.enums.GatewayResultCode;
import org.springframework.stereotype.Component;

import java.nio.charset.Charset;

/**
 * 参数检查过滤器
 *
 * @author key
 */
@Component
public class CheckArgsFilter extends AbstractFilter {

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_3;
    }

    @Override
    public String getFilterName() {
        return "CheckArgsFilter";
    }
    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        // 校验参数非空
        if (StringUtil.isEmpty(request.getAppKey()) || StringUtil.isEmpty(request.getMethod())
            || StringUtil.isEmpty(request.getCharset()) || StringUtil.isEmpty(request.getSignType())
            || StringUtil.isEmpty(request.getSign()) || StringUtil.isEmpty(request.getTimestamp())
            || StringUtil.isEmpty(request.getVersion())
            || StringUtil.isEmpty(request.getParams())) {
            //无效参数
            throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT);
        }

        // 设置默认值
        if (StringUtil.isEmpty(request.getFormat())) {
            request.setFormat(Format.JSON.name());
        }
        try {
            //格式，目前仅支持JSON
            Format.valueOf(request.getFormat());
            //请求使用的编码格式，如UTF-8,GBK,GB2312等
            Charset.forName(request.getCharset());
            //生成签名字符串所使用的签名算法类型
            SignType.valueOf(request.getSignType());
        } catch (Exception ex) {
            //无效参数
            throw new GatewayException(GatewayResultCode.ILLEGAL_ARGUMENT);
        }
        chain.doFilter(request, response);
    }


}
