package com.jz.dm.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.jz.dm.common.exception.GatewayException;
import com.jz.dm.common.util.LogUtil;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.constant.LoggingConstants;
import com.jz.dm.gateway.DefaultOpenApiDispatcher;
import com.jz.dm.models.enity.*;
import com.jz.dm.common.enums.GatewayResultCode;
import com.jz.dm.common.enums.RouteType;
import org.slf4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

/**
 * 网关路由过滤器
 *
 * @author key
 */
@Component("invokeRouteFilter")
public class InvokeRouteFilter extends AbstractFilter {

    /** openapi dispatcher logger */
    private static final Logger DISPATCHER_LOGGER = LoggingConstants.DISPATCHER_LOGGER;

    @Autowired
    DefaultOpenApiDispatcher defaultOpenApiDispatcher;

    @Override
    public int getOrder() {
        return Constants.FILTER_ORDER_6;
    }

    @Override
    public String getFilterName() {
        return "InvokeRouteFilter";
    }

    @Override
    protected void internalDoFilter(GatewayRequest request, GatewayResponse response,
                                    FilterChain chain) {
        String openApiResponse = null;
        //当前系统时间戳
        long startTime = System.currentTimeMillis();
        try {
            OpenApi openApi = (OpenApi) RequestContext.getCurrentContext().get("openApi");
            if (openApi.getRouteType() == null) {
                throw new GatewayException(GatewayResultCode.ILLEGAL_ROUTE_TYPE);
            }
            if (openApi.getRouteType() != RouteType.SRPING) {
                chain.doFilter(request, response);
                return;
            }

            DispatchContext context = new DispatchContext();
            context.setAppKey(request.getAppKey());//apiKey
            context.setApplication(openApi.getApplication());//应用
            //context.setApplication(openApi.getApplication());//应用
            context.setOpenApiMethod(request.getMethod()); //方法 例如:tradd.add
            context.setOpenApiParams(request.getParams());//入参
            context.setOpenApiVersion(request.getVersion()); //版本号
            //扩展参数
            for (GatewayRequest.Attribute attribute : request.getExtAttributes().values()) {
                if (attribute != null && attribute.isPass) {
                    context.setExtAttribute(attribute.name, attribute.value);
                }
            }

            openApiResponse = defaultOpenApiDispatcher.doDispatch(context);

            JSONObject jsonObject = (JSONObject) JSON.parse(openApiResponse);
            response.setAttribute(jsonObject);
        } finally {
            long elapseTime = System.currentTimeMillis() - startTime;
            LogUtil.info(DISPATCHER_LOGGER,
                "gateway do default dispatch,request=" + request + ",response=" + openApiResponse
                                            + "," + elapseTime + "ms");
        }
    }


}
