package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.utils.Result;
import com.jz.dm.config.ReentrantRedisLock;
import com.jz.dm.mapper.ApiReqLogMapper;
import com.jz.dm.models.domian.ApiReqLog;
import com.jz.dm.models.req.LogInfoDetailReq;
import com.jz.dm.models.req.LogInfoListReq;
import com.jz.dm.service.ApiLogService;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiLogServiceImpl
 * @DATE: 2020-12-25/14:29
 * @DAY_NAME_SHORT: 周五
 * @Description:
 **/
@Service("apiLogService")
@Slf4j
public class ApiLogServiceImpl implements ApiLogService {

    @Resource
    private ApiReqLogMapper apiReqLogMapper;
    // 加锁
    @Autowired
    private ReentrantRedisLock lock;
    /**
     * 日志信息列表
     * @param req
     * @return
     */
    @Override
    public IPage<ApiReqLog> listApiLog(LogInfoListReq req) {
        IPage<ApiReqLog> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiReqLog> query = new QueryWrapper<>();
        query.orderByDesc("create_date");
        return apiReqLogMapper.selectPage(page, query);
    }

    /**
     * 查询日志详情
     * @param req
     * @return
     */
    @Override
    public Result getApiLogDetail(LogInfoDetailReq req) {
        return Result.of_success(apiReqLogMapper.selectById(req.getId()));
    }

    /**
     * 保存日志
     * @param reqLog
     */
    @Override
    public void insetLogInfo(ApiReqLog reqLog) {
        apiReqLogMapper.insert(reqLog);
    }


    /**
     * 根据id更新日志
     * @param id
     * @param jsonObject
     */
    @Override
    public void updateLog(Long id, JSONObject jsonObject) {
        //if (!lock.tryLock("apiLog")){
        //  return;
        //}
        try {
            ApiReqLog apiReqLog = apiReqLogMapper.maxId(id);
            if (null != apiReqLog){
                ApiReqLog reqLog = new ApiReqLog();
                reqLog.setId(apiReqLog.getId());
                reqLog.setResponseParams(jsonObject.toString());
                reqLog.setUpdateDate(new Date());
                apiReqLogMapper.updateById(reqLog);
            }else {
                log.info("id为:{}",id+"-----------------日志信息不存在");
            }
        }catch (Exception ex){
            log.error("更新日志返回信息异常:{}",ex.getMessage());
        }
        //finally {
        //    lock.unlock("apiLog");
        //}

    }
}
