package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.Result;
import com.jz.dm.common.constant.Constants;
import com.jz.dm.common.enums.auth.AuthModeEnum;
import com.jz.dm.common.enums.auth.AuthTypeEnum;
import com.jz.dm.common.util.RandomUtil;
import com.jz.dm.mapper.ApiAuthMapper;
import com.jz.dm.mapper.ApiInterfaceMapper;
import com.jz.dm.mapper.ApiOrgMapper;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiOrg;
import com.jz.dm.models.dto.AuthInfoDto;
import com.jz.dm.models.req.auth.*;
import com.jz.dm.models.resp.AuthMallUserResponse;
import com.jz.dm.service.AuthService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: AuthServiceImpl
 * @DATE: 2020-12-23/16:48
 * @DAY_NAME_SHORT: 周三
 * @Description:
 **/
@Service("authService")
@Slf4j
public class AuthServiceImpl implements AuthService {


    @Resource
    private ApiAuthMapper apiAuthMapper;
    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiOrgMapper apiOrgMapper;

    /**
     * 认证信息列表
     *
     * @param req
     * @return
     */
    @Override
    public Result getAuthListInfo(AuthListInfoReq req) {
        IPage<AuthInfoDto> page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<AuthInfoDto> query = new QueryWrapper<>();
        query.orderByDesc("create_date");
        IPage<AuthInfoDto> listSelectApiAuth = apiAuthMapper.listSelectApiAuth(page, query);
        return Result.of_success(listSelectApiAuth);
    }

    /**
     * 认证信息详情
     *
     * @param req
     * @return
     */
    @Override
    public Result getAuthDetailInfo(AuthDetailInfoReq req) {
        return Result.of_success(apiAuthMapper.selectAuthDetail(req));
    }

    /**
     * 添加dmp认证信息
     *
     * @param req
     * @return
     */
    @Override
    public Result addDmpUserApiAuth(AuthDmpUserApiReq req) {
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code",req.getOrgCode()));
        if (null == apiOrg) {
            return Result.of_error("组织信息不存在!");
        }
        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType().name()) &&
                !AuthTypeEnum.DMP_AUTH.name().equals(req.getAuthType())) {
            return Result.of_error("授权类型错误!");
        }
        //认证类型
        if (StringUtils.isNotBlank(req.getAuthMode().name()) && AuthModeEnum.PERMANENT_TIME_MODE.name().equals(req.getAuthMode())) {
            if (StringUtils.isBlank(req.getValidStartTime()) || StringUtils.isBlank(req.getValidEndTime())) {
                return Result.of_error("开始或结束时间不能为空!");
            }
        }
        if (StringUtils.isNotBlank(req.getAuthMode().name()) &&
                AuthModeEnum.POWER_CALL_MODE.name().equals(req.getAuthMode())) {
            return Result.of_error("授权类型错误!");
        }
        String authCode = "";
        ApiAuth apiAuth = new ApiAuth();
        BeanUtils.copyProperties(req, apiAuth);
        apiAuth.setApiInterfaceId(apiInterface.getId());
        if (StringUtils.isNotBlank(req.getOrgType()) //内部组织
                && Constants.AUTH_INT.equalsIgnoreCase(req.getOrgType())) {
            authCode = getAuthCode(apiOrg.getOrgCode(),"",Constants.AUTH_INT);
        }
        /*else if (StringUtils.isNotBlank(req.getOrgType()) //外部组织
                && Constants.AUTH_OUT.equalsIgnoreCase(req.getOrgType())) {
            authCode = getAuthCode("", req.getUserId(),Constants.AUTH_OUT);
        }*/
        apiAuth.setAuthCode(authCode);//授权码
        String salt = RandomUtil.getStringRandom(8);
        apiAuth.setSalt(salt);//盐值
        apiAuth.setStatus("1");//授权状态
        if (apiAuthMapper.insert(apiAuth) > 0) {
            return Result.of_success(ResultMsg.SUCCESS);
        }
        return Result.of_success(ResultMsg.FAILURE);
    }

    /**
     * 添加商城用户认证信息
     *
     * @param req
     * @return
     */
    @Override
    public Result addMallUserApiAuth(AuthMallUserApiReq req) {
        AuthMallUserResponse response = new AuthMallUserResponse();
        ApiInterface apiInterface = getInterface(req.getApiKey());
        if (null == apiInterface) {
            return Result.of_error("api授权信息不存在!");
        }
        ApiOrg apiOrg = apiOrgMapper.selectOne(new QueryWrapper<ApiOrg>().eq("org_code",req.getOrgCode()));
        if (null == apiOrg) {
            return Result.of_error("组织信息不存在!");
        }
        //授权验证
        if (StringUtils.isNotBlank(req.getAuthType().name()) &&
                !AuthTypeEnum.DATA_BANK_AUTH.name().equals(req.getAuthType())) {
            return Result.of_error("授权类型错误!");
        }
        //认证类型
        if (StringUtils.isNotBlank(req.getAuthMode().name()) && AuthModeEnum.RECORD_TIME_MODE.name().equals(req.getAuthMode())) {
            if (StringUtils.isBlank(req.getValidStartTime()) || StringUtils.isBlank(req.getValidEndTime())) {
                return Result.of_error("开始或结束时间不能为空!");
            }
        }
        if (StringUtils.isNotBlank(req.getAuthMode().name()) &&
                AuthModeEnum.PERMANENT_TIME_MODE.name().equals(req.getAuthMode())) {
            return Result.of_error("授权类型错误!");
        }
        String authCode = "";
        ApiAuth apiAuth = new ApiAuth();
        BeanUtils.copyProperties(req, apiAuth);
        apiAuth.setApiInterfaceId(apiInterface.getId());
        if (StringUtils.isNotBlank(req.getOrgType()) //内部组织
                && Constants.AUTH_INT.equalsIgnoreCase(req.getOrgType())) {
            authCode = getAuthCode(apiOrg.getOrgCode(),"",Constants.AUTH_INT);
        } else if (StringUtils.isNotBlank(req.getOrgType()) //外部组织
                && Constants.AUTH_OUT.equalsIgnoreCase(req.getOrgType())) {
            authCode = getAuthCode("", req.getUserId(),Constants.AUTH_OUT);
        }
        apiAuth.setAuthCode(authCode);//授权码
        String salt = RandomUtil.getStringRandom(8);
        apiAuth.setSalt(salt);//盐值
        apiAuth.setStatus("1");//授权状态
        response.setAuthCode(authCode);
        response.setSalt(salt);
        if (apiAuthMapper.insert(apiAuth) > 0) {
            return Result.of_success(ResultMsg.SUCCESS,response);
        }
        return Result.of_success(ResultMsg.FAILURE,response);
    }

    /**
     * 获取授权码
     * @param type
     * @param userId
     * @param apiOrgCode
     * @return
     */
    private String getAuthCode(String apiOrgCode,String userId,String type) {
        String authCode = "";
        authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("auth_code", authCode);
        ApiAuth auth = apiAuthMapper.selectOne(query);
        if (null != auth) {
            authCode = RandomUtil.generateTokenCode(type, apiOrgCode, userId);
        }
        return authCode;
    }

    /**
     * 修改用户认证信息
     *
     * @param req
     * @return
     */
    @Override
    public Result updateAuthInfo(AuthInfoUpdateReq req) {
        //TODO 认证后是否可以修改
        return null;
    }

    /**
     * 盐值重置
     *
     * @param req
     * @return
     */
    @Override
    public Result updateSaltInfo(SaltResetReq req) {
        ApiAuth apiAuth = apiAuthMapper.selectById(req.getId());
        if (null == apiAuth) {
            return Result.of_error("认证用户不存在");
        }
        if (StringUtils.isNotBlank(req.getSalt())) {
            UpdateWrapper<ApiAuth> update = new UpdateWrapper<>();
            update.set("salt", req.getSalt());
            update.set("create_time", new Date());
            update.set("create_user", "");
            update.eq("id", req.getId());
            if (apiAuthMapper.update(null, update) > 0) {
                return Result.of_success(ResultMsg.UPDATE_SUCCESS);
            }
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    private ApiInterface getInterface(String apiKey) {
        QueryWrapper<ApiInterface> queryInface = new QueryWrapper<>();
        queryInface.last("where is_deleted=0 and api_key =" + apiKey + "");
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(queryInface);
        if (null != apiInterface) {
            return apiInterface;
        }
        return null;
    }
}
