package com.jz.dm.web.aspect;


import com.jz.common.utils.IpUtils;
import com.jz.common.utils.JsonUtils;
import com.jz.common.utils.UrlUtil;
import com.jz.dm.common.util.SignType;
import com.jz.dm.models.domian.ApiReqLog;
import com.jz.dm.service.ApiLogService;
import com.jz.dm.web.annotation.ApiLogAspect;
import javassist.*;
import javassist.bytecode.CodeAttribute;
import javassist.bytecode.LocalVariableAttribute;
import javassist.bytecode.MethodInfo;
import lombok.extern.slf4j.Slf4j;
import net.sf.json.JSONObject;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.lang.reflect.Method;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.web.aspect
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: SystemLogAspect
 * @DATE: 2020-12-25/16:03
 * @DAY_NAME_SHORT: 周五
 * @Description:
 **/

@Aspect
@Component
@SuppressWarnings({"unchecked", "unused", "rawtypes"})
@Slf4j
public class SystemLogAspect {

    @Resource
    private ApiLogService apiLogService;

   /* //前置通知切入点
    @Pointcut("@annotation(com.jz.dm.web.annotation.ApiLogAspect)")
    public void beforeAspect() {
    }

    //最终通知切入点
    @Pointcut("@annotation(com.jz.dm.web.annotation.ApiLogAspect)")
    public void lastAspect() {
    }*/

    //环绕通知切入点
    @Pointcut("@annotation(com.jz.dm.web.annotation.ApiLogAspect)")
    public void aroundAspect() {
    }



    @Around("aroundAspect()")
    public void doAround(ProceedingJoinPoint joinPoint) {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = IpUtils.getIpAddr(request);
        //获取用户请求方法的参数并序列化为JSON格式字符串
        String params = "";
        if (joinPoint.getArgs() != null && joinPoint.getArgs().length > 0) {
            for (int i = 0; i < joinPoint.getArgs().length; i++) {
                params += JsonUtils.objectToJson(joinPoint.getArgs()[i]) + ";";
            }
        }
        //获取请求路径
        String url = UrlUtil.getServerUrl(request);
        // 访问项目名
        String contextPath = request.getContextPath();
        log.info("around请求参数为{}", joinPoint.getArgs());
        //动态修改其参数
        //注意，如果调用joinPoint.proceed()方法，则修改的参数值不会生效，必须调用joinPoint.proceed(Object[] args)

        try {
            ApiReqLog reqLog = new ApiReqLog();
            reqLog.setRequestIp(ip);
            // reqLog.setApiKey(jsonObject.getString("apiKey"));
            reqLog.setRequestParams(params);
            reqLog.setResponseParams("");//响应参数
            reqLog.setRequestUrl(url);
            reqLog.setRequestMethod(contextPath);
            reqLog.setEncryMode(SignType.MD5.name()); //加密方式:MD5,RSA
            reqLog.setTransMode(request.getMethod());//传输方式 GET POST
            // reqLog.setRequestToken(token);
            reqLog.setRequestTime(new Date());
            reqLog.setRemark(getServiceMethodDescription(joinPoint));
            System.out.println(reqLog);
            if (null != reqLog) {
                apiLogService.insetLogInfo(reqLog);
            }
            Object result =  joinPoint.proceed(joinPoint.getArgs());
            JSONObject jsonObject = JSONObject.fromObject(result);
            log.info("around响应结果为{}", jsonObject);
            apiLogService.updateLog(reqLog.getId(), jsonObject);
        } catch (Throwable throwable) {
            throwable.printStackTrace();
        }
    }
    /*@Before("beforeAspect()")
    public void doBefore(JoinPoint joinPoint) {
        //日志信息收集切面
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
        String ip = IpUtils.getIpAddr(request);
        //获取用户请求方法的参数并序列化为JSON格式字符串
        String params = "";
        if (joinPoint.getArgs() != null && joinPoint.getArgs().length > 0) {
            for (int i = 0; i < joinPoint.getArgs().length; i++) {
                params += JsonUtils.objectToJson(joinPoint.getArgs()[i]) + ";";
            }
        }
        //获取请求路径
        String url = UrlUtil.getServerUrl(request);
        // 访问项目名
        String contextPath = request.getContextPath();
        //JSONObject jsonObject = JSONObject.parseObject(params);
        //String param = jsonObject.getString("params");
        //JSONObject object = JSONObject.parseObject(param);
        //String token = object.getString("token");
        try {
            log.info("~~~~~~~~~~~~~~~~~~~~~~~前置通知记录请求信息~~~~~~~~~~~~~~~~");
            ApiReqLog reqLog = new ApiReqLog();
            reqLog.setRequestIp(ip);
            // reqLog.setApiKey(jsonObject.getString("apiKey"));
            reqLog.setRequestParams(params);
            reqLog.setResponseParams("");//响应参数
            reqLog.setRequestUrl(url);
            reqLog.setRequestMethod(contextPath);
            reqLog.setEncryMode(SignType.MD5.name()); //加密方式:MD5,RSA
            reqLog.setTransMode(request.getMethod());//传输方式 GET POST
            // reqLog.setRequestToken(token);
            reqLog.setRequestTime(new Date());
            reqLog.setRemark(getServiceMethodDescription(joinPoint));
            System.out.println(reqLog);
            if (null != reqLog) {
                apiLogService.insetLogInfo(reqLog);
            }
        } catch (Exception e) {
            log.error("~~~~~~~~~~~~~~~~~~~~~~~前置通知异常~~~~~~~~~~~~~~~~~~~~~~~");
            log.error("异常信息{}", e.getMessage());
        }

    }*/
    /**
     * 返回异常通知，返回抛出异常的时候执行的通知，可以获得返回的异常
     * 可以访问到异常对象，且可以指定在出现特定异常的时候再执行通知代码
     */
    /*@AfterReturning(value = "lastAspect()", returning = "result")
    public void afterReturn(JoinPoint joinPoint, Object result) {
        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes())
                .getRequest();
        log.info("~~~~~~~~~~~~~~~~~~~~~~最终通知执行~~~~~~~~~~~~~~~~~~~~~~~~");
        String classType = joinPoint.getTarget().getClass().getName();
        try {
            Class<?> clazz = Class.forName(classType);
            String clazzName = clazz.getName();
            // 拦截的方法名称。当前正在执行的方法
            String methodName = joinPoint.getSignature().getName();
            // 获取方法的参数
            Object[] args = joinPoint.getArgs();
            // 获取传入参数的键值对
            Map<String, Object> map = (Map<String, Object>)
                    getFieldsName(this.getClass(), clazzName, methodName,
                            args);
            // 将request中的参数转化为键值对，方便取出
            // Map<String, Object> requestMap = (Map<String, Object>) map.get("request");
            // 获取返回方法的参数
            JSONObject jsonObject = JSONObject.fromObject(result);
            Map mapResult = (Map) jsonObject;
            //将返回的result参数取出
            Map<String, Object> res = (Map<String, Object>) mapResult.get("data");
            Integer id = (Integer) res.get("id");
            JSONObject data = jsonObject.getJSONObject("data");
         //   apiLogService.updateLog(id.longValue(), data);
        } catch (Exception e) {
            log.error("~~~~~~~~~~~~~~~~~~~~~~~最终通知异常~~~~~~~~~~~~~~~~~~~~~~~");
            log.error("异常信息{}", e.getMessage());
        }

    }*/

    /**
     * 获取注解中对方法的描述信息 用于service层注解
     *
     * @param joinPoint
     * @return
     * @throws Exception
     */
    public static String getServiceMethodDescription(JoinPoint joinPoint) throws Exception {
        String targetName = joinPoint.getTarget().getClass().getName();
        String methodName = joinPoint.getSignature().getName();
        Object[] arguments = joinPoint.getArgs();
        Class targetClass = Class.forName(targetName);
        Method[] methods = targetClass.getMethods();
        String description = "";
        for (Method method : methods) {
            if (method.getName().equals(methodName)) {
                Class[] clazzs = method.getParameterTypes();
                if (clazzs.length == arguments.length) {
                    description = method.getAnnotation(ApiLogAspect.class).description();
                    break;
                }
            }
        }
        return description;
    }

    public static Map<String, Object> getFieldsName(Class cls, String clazzName, String methodName, Object[] args) throws Exception {
        Map<String, Object> map = new HashMap<String, Object>();

        ClassPool pool = ClassPool.getDefault();
        ClassClassPath classPath = new ClassClassPath(cls);
        pool.insertClassPath(classPath);

        CtClass cc = pool.get(clazzName);
        CtMethod cm = cc.getDeclaredMethod(methodName);
        MethodInfo methodInfo = cm.getMethodInfo();
        CodeAttribute codeAttribute = methodInfo.getCodeAttribute();
        LocalVariableAttribute attr = (LocalVariableAttribute) codeAttribute.getAttribute(LocalVariableAttribute.tag);
        if (attr == null) {
            // exception
        }
        int pos = Modifier.isStatic(cm.getModifiers()) ? 0 : 1;
        for (int i = 0; i < cm.getParameterTypes().length; i++) {
            map.put(attr.variableName(i + pos), args[i]);// paramNames即参数名
        }
        return map;
    }
}
