package com.jz.manage.moduls.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import com.jz.common.bean.BaseBeanResponse;
import com.jz.common.bean.BaseResponse;
import com.jz.common.bean.PageInfoResponse;
import com.jz.common.bean.SysUserDto;
import com.jz.common.constant.Constants;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultCode;
import com.jz.common.constant.ResultMsg;
import com.jz.common.entity.DataGoods;
import com.jz.common.utils.DateUtils;

import com.jz.common.utils.HttpClientUtils;
import com.jz.common.utils.Result;
import com.jz.common.utils.WebUtils;
import com.jz.manage.moduls.controller.goods.bean.dto.DataGoodsDto;
import com.jz.manage.moduls.controller.goods.bean.dto.DataGoodsListDto;
import com.jz.manage.moduls.controller.goods.bean.request.DataApiUpReq;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsListRequest;
import com.jz.manage.moduls.controller.goods.bean.request.DataGoodsRequest;
import com.jz.manage.moduls.controller.goods.bean.request.MakeDataBankApiReq;
import com.jz.manage.moduls.entity.DataGoodsApi;
import com.jz.manage.moduls.entity.DataGoodsApiParams;
import com.jz.manage.moduls.mapper.DataGoodsApiDao;
import com.jz.manage.moduls.mapper.DataGoodsApiParamsDao;
import com.jz.manage.moduls.mapper.DataGoodsDao;
import com.jz.manage.moduls.service.DataGoodsService;

import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import org.springframework.web.client.RestTemplate;

/**
 * 数据商品(DataGoods)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:30
 */
@Service("dataGoodsService")
public class DataGoodsServiceImpl implements DataGoodsService {
    @Autowired
    private DataGoodsDao dataGoodsDao;

    @Autowired
    private DataGoodsApiParamsDao apiParamsDao;

    @Autowired
    private RedisTemplate redisTemplate;

    @Autowired
    private DataGoodsApiDao tDataGoodsApiDao;

    @Autowired
    private RestTemplate restTemplate;
    /**
     * 条件分页查询数据列表
     *
     * @param goodsListRequest
     * @param httpRequest
     * @return
     * @throws Exception
     */
    @Override
    public PageInfoResponse<DataGoodsListDto> findList(DataGoodsListRequest goodsListRequest, HttpServletRequest httpRequest) throws Exception {
        PageInfoResponse<DataGoodsListDto> pageInfoResponse = new PageInfoResponse<>();

        Map<String, Object> param = new HashMap<>();
        //数据商品id
        if (goodsListRequest.getDataGoodsId() != null) {
            param.put("dataGoodsId", goodsListRequest.getDataGoodsId());
        }
        //商品分类id（行业）
        if (goodsListRequest.getCategoryId() != null) {
            param.put("categoryId", goodsListRequest.getCategoryId());
        }
        //数据商品名称
        if (!StringUtils.isEmpty(goodsListRequest.getDataName())) {
            param.put("dataName", goodsListRequest.getDataName());
        }
        //数据类型：01api，02数据包
        if (!StringUtils.isEmpty(goodsListRequest.getDataType())) {
            param.put("dataType", goodsListRequest.getDataType());
        }
        //数据状态：01预售，02在售中，03下架，04上架
        if (!StringUtils.isEmpty(goodsListRequest.getDataStatus())) {
            param.put("dataStatus", goodsListRequest.getDataStatus());
        }

        PageHelper.startPage(goodsListRequest.getPageNum(), goodsListRequest.getPageSize());
        List<DataGoodsListDto> list = dataGoodsDao.findList(param);
        PageInfo<DataGoodsListDto> pageInfo = new PageInfo<>(list);

        pageInfoResponse.setCode(Constants.SUCCESS_CODE);
        pageInfoResponse.setMessage("查询成功");
        pageInfoResponse.setData(pageInfo);
        return pageInfoResponse;
    }


    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#add(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
//	@Override
//	@Transactional(rollbackFor=Exception.class)
//	public BaseBeanResponse<DataGoods> add(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
//		BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();
//
//		/*$*seDataGoodsIdCodeReplace*$*/
//		//创建人
//		//dataGoods.setCreateUser(getHttpRequestUserId(httpRequest));
//		//创建时间
//		dataGoods.setCreTime(new Date());
//
//		dataGoodsDao.insertSelective(dataGoods);
//
//		baseBeanResponse.setCode(Constants.SUCCESS_CODE);
//		baseBeanResponse.setMessage("新增成功");
//		baseBeanResponse.setData(dataGoods);
//
//		return baseBeanResponse;
//	}

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#edit(com.ycxc.vmts.entity.DataGoods, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseBeanResponse<DataGoods> edit(DataGoods dataGoods, HttpServletRequest httpRequest) throws Exception {
        BaseBeanResponse<DataGoods> baseBeanResponse = new BaseBeanResponse<>();

        //修改人
        //dataGoods.setUpdateUser(getHttpRequestUserId(httpRequest));
        //修改时间
        dataGoods.setUptTime(new Date());

        dataGoodsDao.updateByPrimaryKeySelective(dataGoods);

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("修改成功");
        baseBeanResponse.setData(dataGoods);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#findById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    public BaseBeanResponse<DataGoodsDto> findById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseBeanResponse<DataGoodsDto> baseBeanResponse = new BaseBeanResponse<>();

        DataGoodsDto DataGoodsDto = dataGoodsDao.findById(dataGoodsRequest.getDataGoodsId());

        baseBeanResponse.setCode(Constants.SUCCESS_CODE);
        baseBeanResponse.setMessage("查询成功");
        baseBeanResponse.setData(DataGoodsDto);

        return baseBeanResponse;
    }

    /* (non-Javadoc)
     * @see com.ycxc.vmts.service.DataGoodsService#deleteById(com.ycxc.vmts.controller.bean.DataGoodsRequest, javax.servlet.http.HttpServletRequest)
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public BaseResponse deleteById(DataGoodsRequest dataGoodsRequest, HttpServletRequest httpRequest)
            throws Exception {
        BaseResponse baseResponse = new BaseResponse();

        dataGoodsDao.deleteByPrimaryKey(dataGoodsRequest.getDataGoodsId());

        baseResponse.setCode(Constants.SUCCESS_CODE);
        baseResponse.setMessage("删除成功");

        return baseResponse;
    }


    /**
     * 自营数据-上传api
     *
     * @param dataApiUpReq
     */
    @Override
    @Transactional(rollbackFor = Exception.class)
    public Result uploadApi(DataApiUpReq dataApiUpReq) throws Exception  {
        // 获取当前用户
        SysUserDto user = (SysUserDto) redisTemplate.opsForValue().get("user_" + RedisMessageConstant.SENDTYPE_LOGIN_SYS);

        if (dataApiUpReq == null) {
            return Result.error("上传失败！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getApiKey())) {
            return Result.error("apiKey不存在！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getApiType())) {
            return Result.error("api类型不存在！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getTransMode())) {
            return Result.error("传输方式不能为空！");
        }
        if (!StringUtils.isNotEmpty(dataApiUpReq.getEncryMode())) {
            return Result.error("加密方式不能为空！");
        }
        MakeDataBankApiReq apiReq = new MakeDataBankApiReq();
        DataGoods dataGoods = new DataGoods();
        dataApiUpReq.setApiOpenId("wj"+DateUtils.getHMS());
        dataApiUpReq.setCreTime(DateUtils.getToday());
        dataApiUpReq.setCrePerson(user.getUserId() + "");
        dataApiUpReq.setAuditTime(DateUtils.getToday());
        dataApiUpReq.setAuditPerson(user.getUserId() + "");
        dataApiUpReq.setAuditStatus("02");
        dataApiUpReq.setDataStatus("02");
        BeanUtils.copyProperties(dataApiUpReq, dataGoods);
        dataGoodsDao.insert(dataGoods);

        // 赋值api
        Long dataGoodsId = dataGoods.getDataGoodsId();
        if (dataGoodsId != null) {
            DataGoodsApi dataGoodsApi = new DataGoodsApi();

            dataGoodsApi.setDataGoodsId(dataGoodsId);
            dataApiUpReq.setCreTime(DateUtils.getToday());
            dataApiUpReq.setCrePerson(user.getUserId() + "");
            dataApiUpReq.setEncryMode("MD5");
            dataApiUpReq.setTransMode("http");
            dataApiUpReq.setVersionNumber("1.0");
            BeanUtils.copyProperties(dataApiUpReq, dataGoodsApi);
            tDataGoodsApiDao.insert(dataGoodsApi);

            apiReq.setApiDesc(dataApiUpReq.getApiDesc());
            apiReq.setApiExample("");
            apiReq.setApiFunction(dataApiUpReq.getApiFunction());
            apiReq.setApiId(dataApiUpReq.getApiKey());
            apiReq.setApiType(dataApiUpReq.getApiType());
            apiReq.setVersion(dataApiUpReq.getVersionNumber());
            apiReq.setTransMode(dataApiUpReq.getTransMode());
            apiReq.setType(dataApiUpReq.getEncryMode());
            apiReq.setTargetUrl(dataApiUpReq.getTargetUrl());
            apiReq.setTimeout(dataApiUpReq.getOverTime() + "");
            apiReq.setReqHeaders("sdadasd");
            apiReq.setRequestParam("sdadasdd");
            apiReq.setResponseParam("fklfhklfhlh");
            apiReq.setRespCode("sdkadlahlf");
//            apiReq.setFileId(1L);


            // 赋值params
            Long goodsApi = dataGoodsApi.getGoodsApi();
            if (goodsApi != null) {
                List<DataGoodsApiParams> paramsList = dataApiUpReq.getParamsList();
                for (DataGoodsApiParams params : paramsList) {
                    params.setGoodsApi(goodsApi);
                    dataApiUpReq.setCreTime(DateUtils.getToday());
                    dataApiUpReq.setCrePerson(user.getUserId() + "");
                    BeanUtils.copyProperties(dataApiUpReq, params);
                    apiParamsDao.insert(params);


                }
            }
        }

        try {
            String url= "http://192.168.1.114:8088/api/producer/addDataBankApiInfo";
//            HttpClientUtils.doPost("", );
            WebUtils.postJSON(url, apiReq);
        } catch (Exception e) {
            e.printStackTrace();
            throw e;
        }


//		return Result.of_success(ResultMsg.SUCCESS, ResultCode.SUCCESS);
        return Result.of_success(ResultMsg.SUCCESS, dataApiUpReq);
    }

}